<%--
SDK call returning status of document (BackStage) task generating viewing data for WCR Viewer.
Use it to check (even periodically) whether particular document is prepared for viewing.

author: JANH
--%>
<%@ include file="includes/jspheader.jsp"%>
<%@ include file="includes/jspencoding.jsp"%>
<%@ include file="includes/sdkinit.jsp"%>
<%@ include file="includes/sdklogincheck.jsp"%>

<%@ page import="com.esko.webcenter.jsp.*"%>
<%@ page import="com.esko.webcenter.src.root.DocumentMgrTM"%>
<%@ page import="com.esko.webcenter.src.root.ProjectTM"%>
<%@ page import="com.esko.webcenter.src.root.DocTaskMgrTM"%>
<%@ page import="com.esko.webcenter.src.util.Xml"%>
<%@ page import="com.esko.webcenter.src.xml.DocTaskXml"%>
<%@ page import="com.esko.webcenter.src.util.InvalidParameterException"%>
<%@ page import="com.esko.webcenter.src.root.DocTaskMgrDB"%>
<%@ page import="com.esko.util.StringUtils"%>
<%
	// Start of the main scriplet

	String loginID = LoginMgr.getLoginMgr(session).getLoginID();

	// get parameters
	Properties requestParams = SDKUtils
			.getRequestParams(request, session, application, requestParamsLoginCheck);

	String docID = (requestParams.containsKey("documentid")) ? ((String[]) requestParams.get("documentid"))[0]
			: "";
	String docVerID = (requestParams.containsKey("docversionid")) ? ((String[]) requestParams
			.get("docversionid"))[0] : "";
	String projectID = (requestParams.containsKey("projectid")) ? ((String[]) requestParams.get("projectid"))[0]
			: "";
	String projectName = (requestParams.containsKey("projectname")) ? ((String[]) requestParams
			.get("projectname"))[0] : "";
	String docName = (requestParams.containsKey("documentname")) ? ((String[]) requestParams
			.get("documentname"))[0] : "";

	// create the EJBs
	DocumentMgrTM documentMgrTM = DBTMUtils.findBean(response, session, DocumentMgrTM.class);
	ProjectTM projectTM = DBTMUtils.findBean(response, session, ProjectTM.class);
	DocTaskMgrTM docTaskMgrTM = DBTMUtils.findBean(response, session, DocTaskMgrTM.class);

	if ((documentMgrTM == null) || (projectTM == null) || (docTaskMgrTM == null)) {
		out.write(SDKUtils.ShowError(SDKUtils.SDK_UNDEF_ERROR));
		checkLogOff(session);
		return;
	}

	StringBuffer errorMsg = new StringBuffer();

	// find out specified document version
	DocVersion docVersion = null;

	if (StringUtils.checkForValue(docVerID) || StringUtils.checkForValue(docID)) {
		// document or doc version ID specified - load the latest version of the document
		docVersion = SDKUtils.getDocVersionByIDOrDocID(loginID, docVerID, docID, errorMsg, documentMgrTM);
	} else if ((StringUtils.checkForValue(projectID) || StringUtils.checkForValue(projectName))
			&& StringUtils.checkForValue(docName)) {
		// project ID or name and document name specified - load the latest version of the document based on that
		docVersion = SDKUtils.getDocVersionByDocNameProjectIDOrName(loginID, docName, projectID, projectName,
				errorMsg, documentMgrTM, projectTM);
	}

	if (docVersion == null) {
		out.write(errorMsg.toString());
		checkLogOff(session);
		return;
	}

	// having document version object, load document task to get its status
	DocTask docTask = null;

	if (DocTypeIdUtils.isDocTaskStatusInfoRequired(docVersion.getDocTypeID())) {
		try {
			docTask = docTaskMgrTM.loadDocVersionTaskStatus(loginID, docVersion.getObjectID());
		} catch (Exception ex) {
			docTask = null;
		}
		if (docTask == null) {
			out.write(SDKUtils.ShowError(SDKUtils.SDK_DOC_TASK_LOAD_FAILED));
			checkLogOff(session);
			return;
		}
	}

	boolean docViewable = (docTask != null);
	boolean docReadyToView = docViewable
			&& docTask.getStatusID().equalsIgnoreCase(DocTaskMgrDB.StatusTypes.success);
	boolean dataGenInProgress = docViewable
			&& docTask.getStatusID().equalsIgnoreCase(DocTaskMgrDB.StatusTypes.started);
	
	
	// Bugfix: following code is a correction on the cache implemented on the filestore: 
	// When getting a view filespec, the code on the backend uses a cache on the filestore content
	// which will keep the value in the cache for 1 minute. This can lead to a descrepency between 
	// the database table (used to store the status of the view file task), and the actual behavior of the viewer, 
	// in the sense that the filestore can return 0 (== nu view file) while the document just became viewable. 
	// To make these two inline, we will verify if the viewer can indeed be opened when the viewer status returns 1.    
	if (docReadyToView && DocTypeIdUtils.isViewableGraphics(docVersion.getDocTypeID()) ){
		OptimizedDocWriter optimizedDocWriter = DBTMUtils.findBean(response, session, OptimizedDocWriter.class);
		String viewFile = optimizedDocWriter.GetSingleRelativeViewDataURL(docVersion.getObjectID());
		if (!StringUtils.checkForValue(viewFile)){
			docReadyToView = false;
			// update the task record to be inline with the status
			docTask.setFinishedDate((String)null);
			docTask.setStatusID("00001_0000000002");
		}
	}

	// generate output XML
	try {
		out.write(Xml.getBeginTag("result"));
		out.write(Xml.getFieldXml("doc_supported", docViewable));
		out.write(Xml.getFieldXml("doc_ready_to_view", docReadyToView));
		out.write(Xml.getFieldXml("data_generation_in_progress", dataGenInProgress));
		if (docViewable) {
			DocTaskXml docTaskXml = new DocTaskXml();
			out.write(docTaskXml.getInXml(docTask));
		}
		out.write(Xml.getEndTag("result"));
	} catch (XmlException xmlEx) {
		out.write(SDKUtils.ShowError(SDKUtils.SDK_UNDEF_ERROR, xmlEx));
	} catch (InvalidParameterException invalidParmaEx) {
		out.write(SDKUtils.ShowError(SDKUtils.SDK_UNDEF_ERROR, invalidParmaEx));
	} finally {
		checkLogOff(session);
	}
%>
